﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Core;
using HeuristicLab.Encodings.PermutationEncoding;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Data;
using System.Collections.Generic;

namespace HeuristicLab.Problems.VehicleRouting.Encodings.GVR {
  [Item("GVRDisplacementManipulator", "An operator which manipulates a GVR representation by applying a displacement operation. ")]
  [StorableClass]
  public sealed class GVRDisplacementManipulator : GVRManipulator {
    [StorableConstructor]
    private GVRDisplacementManipulator(bool deserializing) : base(deserializing) { }

    public GVRDisplacementManipulator()
      : base() {
    }

    protected override void Manipulate(IRandom random, GVREncoding individual) {
      Tour tour = individual.Tours[random.Next(individual.Tours.Count)];
      int breakPoint1 = random.Next(tour.Cities.Count);
      int length = random.Next(1, tour.Cities.Count - breakPoint1 + 1);

      List<int> displaced = tour.Cities.GetRange(breakPoint1, length);
      tour.Cities.RemoveRange(breakPoint1, length);
      //with a probability of 1/(2*V) create a new tour, else insert at another position
      if (individual.GetTours().Count < VehiclesParameter.ActualValue.Value && 
        random.Next(individual.GetTours().Count * 2) == 0) {
        Tour newTour = new Tour();
        newTour.Cities.InsertRange(0, displaced);

        individual.Tours.Add(newTour);
      } else {
        Tour newTour = individual.Tours[random.Next(individual.Tours.Count)];
        int newPosition = newTour.Cities.Count;

        newTour.Cities.InsertRange(newPosition, displaced);
      }

      if (tour.Cities.Count == 0)
        individual.Tours.Remove(tour);
    }
  }
}
